@@ -21,7 +21,6 @@ void compilerCallback(int, char*, int, char*, void*);
 import "C"
 import (
 	"errors"
-	"os"
 	"runtime"
 	"unsafe"
 )
@@ -100,39 +99,15 @@ func (c *Compiler) Destroy() {
 	}
 }
 
-// AddFile compiles rules from a file. Rules are added to the
-// specified namespace.
-func (c *Compiler) AddFile(file *os.File, namespace string) (err error) {
-	fd := C.dup(C.int(file.Fd()))
-	fh, err := C.fdopen(fd, C.CString("r"))
-	if err != nil {
-		return err
-	}
-	defer C.fclose(fh)
-	var ns *C.char
-	if namespace != "" {
-		ns = C.CString(namespace)
-		defer C.free(unsafe.Pointer(ns))
-	}
-	filename := C.CString(file.Name())
-	defer C.free(unsafe.Pointer(filename))
-	id := callbackData.Put(c)
-	defer callbackData.Delete(id)
-	C.yr_compiler_set_callback(c.cptr, C.YR_COMPILER_CALLBACK_FUNC(C.compilerCallback), id)
-	numErrors := int(C.yr_compiler_add_file(c.cptr, fh, ns, filename))
-	if numErrors > 0 {
-		var buf [1024]C.char
-		msg := C.GoString(C.yr_compiler_get_error_message(
-			c.cptr, (*C.char)(unsafe.Pointer(&buf[0])), 1024))
-		err = errors.New(msg)
-	}
-	keepAlive(c)
-	return
-}
-
 // AddString compiles rules from a string. Rules are added to the
 // specified namespace.
+//
+// If this function returns an error, the Compiler object will become
+// unusable.
 func (c *Compiler) AddString(rules string, namespace string) (err error) {
+	if c.cptr.errors != 0 {
+		return errors.New("Compiler cannot be used after parse error")
+	}
 	var ns *C.char
 	if namespace != "" {
 		ns = C.CString(namespace)
@@ -188,6 +163,9 @@ func (c *Compiler) DefineVariable(identifier string, value interface{}) (err err
 
 // GetRules returns the compiled ruleset.
 func (c *Compiler) GetRules() (*Rules, error) {
+	if c.cptr.errors != 0 {
+		return nil, errors.New("Compiler cannot be used after parse error")
+	}
 	var yrRules *C.YR_RULES
 	if err := newError(C.yr_compiler_get_rules(c.cptr, &yrRules)); err != nil {
 		return nil, err
@@ -205,6 +183,7 @@ func Compile(rules string, variables map[string]interface{}) (r *Rules, err erro
 	if c, err = NewCompiler(); err != nil {
 		return
 	}
+	defer c.Destroy()
 	for k, v := range variables {
 		if err = c.DefineVariable(k, v); err != nil {
 			return
