@@ -48,10 +48,6 @@ type MatchString struct {
 	Data   []byte
 }
 
-func init() {
-	_ = C.yr_initialize()
-}
-
 // ScanFlags are used to tweak the behavior of Scan* functions.
 type ScanFlags int
 
@@ -82,7 +78,9 @@ func (r *Rules) ScanMemWithCallback(buf []byte, flags ScanFlags, timeout time.Du
 	if len(buf) > 0 {
 		ptr = (*C.uint8_t)(unsafe.Pointer(&(buf[0])))
 	}
-	id := callbackData.Put(cb)
+	cbc := &scanCallbackContainer{ScanCallback: cb}
+	defer cbc.destroy()
+	id := callbackData.Put(cbc)
 	defer callbackData.Delete(id)
 	err = newError(C.yr_rules_scan_mem(
 		r.cptr,
@@ -111,7 +109,9 @@ func (r *Rules) ScanFile(filename string, flags ScanFlags, timeout time.Duration
 func (r *Rules) ScanFileWithCallback(filename string, flags ScanFlags, timeout time.Duration, cb ScanCallback) (err error) {
 	cfilename := C.CString(filename)
 	defer C.free(unsafe.Pointer(cfilename))
-	id := callbackData.Put(cb)
+	cbc := &scanCallbackContainer{ScanCallback: cb}
+	defer cbc.destroy()
+	id := callbackData.Put(cbc)
 	defer callbackData.Delete(id)
 	err = newError(C.yr_rules_scan_file(
 		r.cptr,
@@ -137,7 +137,9 @@ func (r *Rules) ScanProc(pid int, flags ScanFlags, timeout time.Duration) (match
 // every event emitted by libyara, the appropriate method on the
 // ScanCallback object is called.
 func (r *Rules) ScanProcWithCallback(pid int, flags ScanFlags, timeout time.Duration, cb ScanCallback) (err error) {
-	id := callbackData.Put(cb)
+	cbc := &scanCallbackContainer{ScanCallback: cb}
+	defer cbc.destroy()
+	id := callbackData.Put(cbc)
 	defer callbackData.Delete(id)
 	err = newError(C.yr_rules_scan_proc(
 		r.cptr,
